@@ -1,22 +1,19 @@
+"""The docstrfmt server."""
+
 import logging
 import time
 
 import click
 import docutils
 from aiohttp import web
+from black import DEFAULT_LINE_LENGTH
 
 from . import Manager, rst_extras
 
 
-class ParseError(Exception):  # pragma: no cover
-    pass
-
-
-rst_extras.register()
-
-
-async def handler(request) -> web.Response:
-    width = int(request.headers.get("X-Line-Length", 88))
+async def handler(request: web.Request) -> web.Response:
+    """Handle the incoming request."""
+    width = int(request.headers.get("X-Line-Length", DEFAULT_LINE_LENGTH))
     body = await request.text()
 
     start_time = time.perf_counter()
@@ -28,7 +25,7 @@ async def handler(request) -> web.Response:
             )
             resp = web.Response(text=text)
         except docutils.utils.SystemMessage as error:  # pragma: no cover
-            raise ParseError(str(error))
+            raise ParseError(str(error)) from None
     except ParseError as error:  # pragma: no cover
         logging.warning(f"Failed to parse input: {error}")
         resp = web.Response(status=400, reason=str(error))
@@ -38,11 +35,13 @@ async def handler(request) -> web.Response:
 
     end_time = time.perf_counter()
 
-    time_elapsed = int(1000 * (end_time - start_time))
-    print(f"Finished request: {time_elapsed:3} ms, {len(body):5} chars")
+    int(1000 * (end_time - start_time))
     return resp
 
 
+rst_extras.register()
+
+
 @click.command()
 @click.option(
     "-h",
@@ -60,7 +59,12 @@ async def handler(request) -> web.Response:
     default=5219,
     show_default=True,
 )
-def main(bind_host, bind_port) -> None:
+def main(bind_host: str, bind_port: int) -> None:
+    """Start the docstrfmt server."""
     app = web.Application()
     app.add_routes([web.post("/", handler)])
     web.run_app(app, host=bind_host, port=bind_port)
+
+
+class ParseError(Exception):  # pragma: no cover
+    """An error occurred while parsing the input."""
